import requests  
import numpy as np 
import pandas as pd
from PIL import Image
from transformers import ViTImageProcessor, ViTModel
from io import BytesIO
import torch
import os
import logging


logging.getLogger("transformers.modeling_utils").setLevel(logging.ERROR)


image_url = "https://sbs-bell.com/archive/sbs_logo.png"
response = requests.get(image_url)
if response.status_code == 200:
    img = Image.open(BytesIO(response.content)).convert("RGB")
else:
    raise Exception(f"Failed to download image. Status code: {response.status_code}")

# Initialize image processor and model
model_name = "google/vit-base-patch16-224"
image_processor_name = "google/vit-base-patch16-224"

# Set the device (CPU, MPS, or CUDA)
if torch.backends.mps.is_available():
    device = torch.device('mps')
elif torch.cuda.is_available():
    device = torch.device('cuda:0')
else:
    device = torch.device('cpu')

# Load Vision Transformer model and image processor
image_processor = ViTImageProcessor.from_pretrained(image_processor_name)
model = ViTModel.from_pretrained(model_name).to(device)
inputs = image_processor(images=img, return_tensors="pt").to(device)
# disable gradients since we are in inference mode
with torch.no_grad():
    # ** unpacks the image processor output dictionary into keyword arguments
    outputs = model(**inputs)
embedding = outputs.last_hidden_state[:, 0, :].cpu().numpy()

# jumping through hoops to make sure you can manage python packages
df = pd.DataFrame(embedding.flatten(), columns=["sbs_logo_embedding"])
home_directory = os.path.expanduser("~")

csv_file_path = os.path.join(home_directory, "sbs_logo_embedding.csv")
df.to_csv(csv_file_path, index=False)

print(f"CSV file saved at: {csv_file_path}")
